(function() {
  var DEFAULT, DEFAULT_JOIN, JOIN_TYPES, PLACEHOLDER, common, fields, group, limit, order, renderBoundParam, renderClause, renderInsertParams, set, tables, where, _ref;
  var __indexOf = Array.prototype.indexOf || function(item) {
    for (var i = 0, l = this.length; i < l; i++) {
      if (this[i] === item) return i;
    }
    return -1;
  };
  common = require('./common');
  _ref = ['JOIN_TYPES', 'DEFAULT'].map(function(k) {
    return common[k];
  }), JOIN_TYPES = _ref[0], DEFAULT = _ref[1];
  PLACEHOLDER = '?';
  DEFAULT_JOIN = JOIN_TYPES.INNER;
  exports.pre = {
    orderBy: {
      "ORDER BY with multiple tables is only allowed for SELECT": function() {
        return this.queryType === 'Select' || this.tableStack.length === 1;
      }
    },
    limit: {
      "LIMIT with multiple tables is only allowed for SELECT": function() {
        return this.queryType === 'Select' || this.tableStack.length === 1;
      }
    },
    join: {
      "JOIN with ORDER BY or LIMIT is only allowed for SELECT": function() {
        return this.queryType === 'Select' || (this.order.length === 0 && !this.limit);
      }
    }
  };
  exports.renderSelect = function(qs) {
    return ("SELECT " + (fields(qs)) + " FROM ") + [tables, where, group, order, limit].map(function(f) {
      return f(qs);
    }).join('');
  };
  exports.renderUpdate = function(qs) {
    var parts;
    parts = qs.tableStack.length === 1 ? [tables, set, where, order, limit] : [tables, set, where];
    return "UPDATE " + parts.map(function(f) {
      return f(qs);
    }).join('');
  };
  set = function(qs) {
    return ' SET ' + qs.fields.map(function(f) {
      if (f.match(/\=/)) {
        return f;
      } else {
        return "" + f + " = ?";
      }
    }).join(', ');
  };
  exports.renderInsert = function(qs) {
    return "INSERT INTO " + qs.table + " (" + (qs.fields.join(', ')) + ") VALUES " + (renderInsertParams(qs));
  };
  exports.renderInsertSelect = function(qs) {
    return "INSERT INTO " + table + " " + (qs.fromQuery.toSql());
  };
  exports.renderDelete = function(qs) {
    var parts;
    parts = qs.tableStack.length === 1 ? [tables, where, order, limit] : [tables, where];
    return "DELETE FROM " + parts.map(function(f) {
      return f(qs);
    }).join('');
  };
  fields = function(qs) {
    var f, fs, tbl, tbl_fields, _i, _len, _ref2;
    fs = [];
    _ref2 = qs.fields;
    for (tbl in _ref2) {
      tbl_fields = _ref2[tbl];
      if (tbl_fields == null) {
        continue;
      }
      if (tbl_fields.length) {
        for (_i = 0, _len = tbl_fields.length; _i < _len; _i++) {
          f = tbl_fields[_i];
          fs.push(f[0] === f[1] ? "" + tbl + "." + f[0] : "" + tbl + "." + f[0] + " AS '" + f[1] + "'");
        }
      } else {
        fs.push("" + tbl + ".*");
      }
    }
    return fs.join(', ');
  };
  tables = exports.tables = function(qs) {
    var alias, clause, i, ret, table, ts, type;
    i = 0;
    ts = (function() {
      var _i, _len, _ref2, _ref3, _results;
      _ref2 = qs.tableStack;
      _results = [];
      for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
        _ref3 = _ref2[_i], table = _ref3[0], alias = _ref3[1], type = _ref3[2], clause = _ref3[3];
        if (type === 'NOP') {
          continue;
        }
        ret = i++ ? "" + (type.toUpperCase()) + " JOIN " + table : table;
        if (table !== alias) {
          ret += " AS " + alias;
        }
        if (clause != null) {
          ret += " ON " + (renderClause(clause, function(v) {
            return v;
          }));
        }
        _results.push(ret);
      }
      return _results;
    })();
    return ts.join(' ');
  };
  where = exports.where = function(qs) {
    if (qs.where.length) {
      return " WHERE " + (renderClause(qs.where));
    } else {
      return "";
    }
  };
  group = exports.group = function(qs) {
    if (qs.groupings.length) {
      return " GROUP BY " + (qs.groupings.map(function(g) {
        return g.table + '.' + g.field;
      }).join(', '));
    } else {
      return "";
    }
  };
  order = exports.order = function(qs) {
    if (qs.order.length) {
      return " ORDER BY " + (qs.order.map(function(o) {
        return o.table + '.' + o.field + (o.direction ? ' ' + o.direction : '');
      }).join(', '));
    } else {
      return "";
    }
  };
  limit = function(qs) {
    if (qs.limit != null) {
      return " LIMIT " + qs.limit;
    } else {
      return "";
    }
  };
  renderBoundParam = function(v) {
    if (v && v.constructor === Array) {
      return "(" + (v.map(function() {
        return PLACEHOLDER;
      }).join(', ')) + ")";
    } else {
      return PLACEHOLDER;
    }
  };
  exports.renderClause = renderClause = function(input, renderValue) {
    var render;
    if (renderValue == null) {
      renderValue = renderBoundParam;
    }
    render = function(clause) {
      if ((clause != null) && clause.constructor === Array) {
        return "" + (clause.map(render).join(' AND '));
      } else if (typeof clause === 'object') {
        if (clause.op === 'multi') {
          return "(" + (clause.clauses.map(render).join(clause.glue)) + ")";
        } else {
          return "" + clause.table + "." + clause.field + " " + clause.op + " " + (renderValue(clause.value));
        }
      } else {
        throw new Error("Unexpected clause type, this is probably a bug");
      }
    };
    return render(input);
  };
  exports.joinOp = exports.whereOp = function(op) {
    switch (op.toLowerCase()) {
      case 'ne':
      case '!=':
      case '<>':
        return '!=';
      case 'eq':
      case '=':
        return '=';
      case 'lt':
      case '<':
        return '<';
      case 'gt':
      case '>':
        return '>';
      case 'lte':
      case '<=':
        return '<=';
      case 'gte':
      case '>=':
        return '>=';
      case 'like':
        return 'LIKE';
      case 'in':
        return 'IN';
      default:
        throw new Error("Unsupported comparison operator: " + op);
    }
  };
  exports.joinType = function(type) {
    if (!type) {
      return 'INNER';
    }
    type = type.toUpperCase();
    if (__indexOf.call(JOIN_TYPES, type) >= 0) {
      return type;
    } else {
      throw new Error("Unsupported JOIN type " + type);
    }
  };
  renderInsertParams = function(qs) {
    var rows;
    rows = (function() {
      var _i, _j, _ref2, _ref3, _results, _results2;
      _results = [];
      for (_i = 1, _ref2 = qs.parameters.length / qs.fields.length; 1 <= _ref2 ? _i <= _ref2 : _i >= _ref2; 1 <= _ref2 ? _i++ : _i--) {
        _results.push(renderBoundParam((function() {
          _results2 = [];
          for (var _j = 1, _ref3 = qs.fields.length; 1 <= _ref3 ? _j <= _ref3 : _j >= _ref3; 1 <= _ref3 ? _j++ : _j--){ _results2.push(_j); }
          return _results2;
        }).apply(this)));
      }
      return _results;
    })();
    return rows.join(", ");
  };
}).call(this);
